@@ -1,7 +1,7 @@
 /*
 Copyright 2008, 2009 Will Stephenson <wstephenson@kde.org>
 Copyright 2008, 2009 Sebastian Kügler <sebas@kde.org>
-Copyright 2011-2012 Lamarque V. Souza <lamarque@kde.org>
+Copyright 2011-2013 Lamarque V. Souza <lamarque@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -273,7 +273,7 @@ void NetworkManagerApplet::setupInterfaceSignals()
             Solid::Control::ModemGsmNetworkInterface *modemNetworkIface = modemiface->getModemNetworkIface();
             if (modemNetworkIface) {
                 connect(modemNetworkIface, SIGNAL(signalQualityChanged(uint)), this, SLOT(interfaceConnectionStateChanged()));
-                connect(modemNetworkIface, SIGNAL(accessTechnologyChanged(Solid::Control::ModemNetworkInterfaceNm09::AccessTechnology)), this, SLOT(interfaceConnectionStateChanged()));
+                connect(modemNetworkIface, SIGNAL(accessTechnologyChanged(Solid::Control::ModemInterface::AccessTechnology)), this, SLOT(interfaceConnectionStateChanged()));
             }
         }
     }
@@ -1125,4 +1125,9 @@ void NetworkManagerApplet::updateInterfaceList()
     setStatus(m_interfaces.isEmpty() ? Plasma::PassiveStatus : Plasma::ActiveStatus);
 }
 
+void NetworkManagerApplet::popupEvent(bool show)
+{
+    m_popup->setPopupOpened(show);
+}
+
 #include "networkmanager.moc"
