@@ -1,7 +1,7 @@
 /*
 Copyright 2008,2009 Will Stephenson <wstephenson@kde.org>
 Copyright 2008-2010 Sebastian Kügler <sebas@kde.org>
-Copyright 2011-2012 Lamarque V. Souza <lamarque@kde.org>
+Copyright 2011-2013 Lamarque V. Souza <lamarque@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -66,6 +66,7 @@ NMPopup::NMPopup(RemoteActivatableList * activatableList, QGraphicsWidget* paren
     m_hasWirelessInterface(false),
     m_widget(0),
     m_mainLayout(0),
+    m_warning(0),
     m_leftWidget(0),
     m_leftLayout(0),
     m_interfaceLayout(0),
@@ -216,8 +217,8 @@ void NMPopup::init()
     m_showMoreButton->setMinimumSize(sMax);
     m_showMoreButton->setMaximumSize(sMax);
     connect(m_showMoreButton, SIGNAL(clicked()), this, SLOT(showMore()));
-    connect(m_activatables, SIGNAL(activatableAdded(RemoteActivatable *, int)), this, SLOT(uncheckShowMore(RemoteActivatable *)));
-    connect(m_activatables, SIGNAL(activatableRemoved(RemoteActivatable *)), this, SLOT(checkShowMore(RemoteActivatable *)));
+    connect(m_activatables, SIGNAL(activatableAdded(RemoteActivatable*,int)), this, SLOT(uncheckShowMore(RemoteActivatable*)));
+    connect(m_activatables, SIGNAL(activatableRemoved(RemoteActivatable*)), this, SLOT(checkShowMore(RemoteActivatable*)));
 
     QGraphicsLinearLayout* connectionLayout = new QGraphicsLinearLayout;
     connectionLayout->addItem(m_showMoreButton);
@@ -236,10 +237,10 @@ void NMPopup::init()
     }
     addVpnInterface();
     // hook up signals to allow us to change the connection list depending on APs present, etc
-    connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceAdded(const QString&)),
-            SLOT(interfaceAdded(const QString&)));
-    connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceRemoved(const QString&)),
-            SLOT(interfaceRemoved(const QString&)));
+    connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceAdded(QString)),
+            SLOT(interfaceAdded(QString)));
+    connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceRemoved(QString)),
+            SLOT(interfaceRemoved(QString)));
 
     m_showMoreChecked = false;
     m_oldShowMoreChecked = true;
@@ -259,34 +260,6 @@ void NMPopup::init()
     adjustSize();
 }
 
-static int compareVersions(const QString & version1, const QString & version2)
-{
-    QStringList sl1 = version1.split('.');
-    QStringList sl2 = version2.split('.');
-
-    if (sl1.size() > 2 && sl2.size() > 2) {
-        int v1[3] = { sl1[0].toInt(), sl1[1].toInt(), sl1[2].toInt() };
-        int v2[3] = { sl2[0].toInt(), sl2[1].toInt(), sl2[2].toInt() };
-
-        if (v1[0] > v2[0]) {
-            return 1;
-        } else if (v1[0] < v2[0]) {
-            return -1;
-        } else if (v1[1] > v2[1]) {
-            return 1;
-        } else if (v1[1] < v2[1]) {
-            return -1;
-        } else if (v1[2] > v2[2]) {
-            return 1;
-        } else if (v1[2] < v2[2]) {
-            return -1;
-        } else {
-            return 0;
-        }
-    }
-    return 0;
-}
-
 void NMPopup::readConfig()
 {
     kDebug();
@@ -315,13 +288,20 @@ void NMPopup::readConfig()
 
     QString version = Solid::Control::NetworkManagerNm09::version();
     if (version.isEmpty()) {
-        Plasma::Label * warning = new Plasma::Label(this);
-        warning->setText(i18nc("Warning about wrong NetworkManager version", "NetworkManager is not running. Please start it."));
-        m_interfaceLayout->addItem(warning);
-    } else if (compareVersions(version, QString(MINIMUM_NM_VERSION_REQUIRED)) < 0) {
-        Plasma::Label * warning = new Plasma::Label(this);
-        warning->setText(i18nc("Warning about wrong NetworkManager version", "We need at least NetworkManager-%1 to work properly, found '%2'. Please upgrade to a newer version.", QString(MINIMUM_NM_VERSION_REQUIRED), version));
-        m_interfaceLayout->addItem(warning);
+        if (!m_warning) {
+            m_warning = new Plasma::Label(this);
+        }
+        m_warning->setText(i18nc("Warning about wrong NetworkManager version", "NetworkManager is not running. Please start it."));
+        m_interfaceLayout->addItem(m_warning);
+    } else if (Solid::Control::NetworkManagerNm09::compareVersion(QString(MINIMUM_NM_VERSION_REQUIRED)) < 0) {
+        if (!m_warning) {
+            m_warning = new Plasma::Label(this);
+        }
+        m_warning->setText(i18nc("Warning about wrong NetworkManager version", "We need at least NetworkManager-%1 to work properly, found '%2'. Please upgrade to a newer version.", QString(MINIMUM_NM_VERSION_REQUIRED), version));
+        m_interfaceLayout->addItem(m_warning);
+    } else if (m_warning) {
+        m_warning->deleteLater();
+        m_warning = 0;
     }
 }
 
@@ -414,17 +394,17 @@ void NMPopup::addInterfaceInternal(Solid::Control::NetworkInterfaceNm09* iface)
             //connect(wirelessinterface, SIGNAL(stateChanged()), this, SLOT(updateNetworks()));
             wifiItem->setEnabled(Solid::Control::NetworkManagerNm09::isWirelessEnabled());
             kDebug() << "WiFi added";
-            connect(wifiItem, SIGNAL(disconnectInterfaceRequested(const QString&)), m_connectionList, SLOT(deactivateConnection(const QString&)));
+            connect(wifiItem, SIGNAL(disconnectInterfaceRequested(QString)), m_connectionList, SLOT(deactivateConnection(QString)));
         } else {
             // Create the interfaceitem
             ifaceItem = new InterfaceItem(static_cast<Solid::Control::WiredNetworkInterfaceNm09 *>(iface), m_activatables, InterfaceItem::InterfaceName, this);
-            connect(ifaceItem, SIGNAL(disconnectInterfaceRequested(const QString&)), m_connectionList, SLOT(deactivateConnection(const QString&)));
+            connect(ifaceItem, SIGNAL(disconnectInterfaceRequested(QString)), m_connectionList, SLOT(deactivateConnection(QString)));
         }
         connect(ifaceItem, SIGNAL(clicked()), this, SLOT(toggleInterfaceTab()));
         connect(ifaceItem, SIGNAL(clicked(Solid::Control::NetworkInterfaceNm09*)),
                 m_connectionList,  SLOT(addInterface(Solid::Control::NetworkInterfaceNm09*)));
-        connect(ifaceItem, SIGNAL(hoverEnter(const QString&)), m_connectionList, SLOT(hoverEnter(const QString&)));
-        connect(ifaceItem, SIGNAL(hoverLeave(const QString&)), m_connectionList, SLOT(hoverLeave(const QString&)));
+        connect(ifaceItem, SIGNAL(hoverEnter(QString)), m_connectionList, SLOT(hoverEnter(QString)));
+        connect(ifaceItem, SIGNAL(hoverLeave(QString)), m_connectionList, SLOT(hoverLeave(QString)));
 
         // Catch connection changes
         connect(iface, SIGNAL(connectionStateChanged(int,int,int)), this, SLOT(handleConnectionStateChange(int,int,int)));
@@ -520,6 +500,11 @@ void NMPopup::wwanEnabledToggled(bool checked)
 
 void NMPopup::networkingEnabledToggled(bool checked)
 {
+    if (checked && m_warning) {
+        m_warning->deleteLater();
+        m_warning = 0;
+    }
+
     // Switch networking on / off
     if (Solid::Control::NetworkManagerNm09::isNetworkingEnabled() != checked) {
         Solid::Control::NetworkManagerNm09::setNetworkingEnabled(checked);
@@ -752,6 +737,11 @@ void NMPopup::toggleInterfaceTab()
     }
 }
 
+void NMPopup::setPopupOpened(bool opened)
+{
+    m_interfaceDetailsWidget->setUpdateEnabled(opened && m_leftWidget->currentIndex() == 1);
+}
+
 void NMPopup::refresh()
 {
     m_interfaceDetailsWidget->show();
